from __future__ import print_function
from artemis.plotting.plotting_backend import set_server_plotting
from collections import namedtuple
import Queue
import argparse
import atexit
import sys
import os
import threading
import time
import pickle
from datetime import datetime
import signal
from artemis.fileman.local_dir import get_local_path, format_filename
from artemis.plotting.db_plotting import dbplot, hold_dbplots, set_dbplot_figure_size
from artemis.plotting.saving_plots import save_figure
from artemis.remote.utils import get_socket, recv_size, send_size
from matplotlib import pyplot as plt

sys.path.extend([os.path.dirname(os.path.dirname(__file__))])


def send_port_if_running_and_join():
    port_file_path = get_local_path("tmp/plot_server/port.info", make_local_dir=True)
    if os.path.exists(port_file_path):
        with open(port_file_path, 'r') as f:
            port = pickle.load(f)
        print(port)
        print("Your dbplot call is attached to an existing plotting server. \nAll stdout and stderr of this existing plotting server "
              "is forwarded to the process that first created this plotting server. \nIn the future we might try to hijack this and provide you "
              "with these data streams")
        print("Use with care, this functionallity might have unexpected side issues")
        try:
            while(True):
                time.sleep(20)
        except KeyboardInterrupt:
            print(" Redirected Server killed")
            sys.exit()
    else:
        with open(port_file_path,"w") as f:
            pass


def write_port_to_file(port):
    atexit.register(remove_port_file)
    port_file_path = get_local_path("tmp/plot_server/port.info", make_local_dir=True)
    if os.path.exists(port_file_path):
        print("port.info file already exists. This might either mean that you are running another plotting server in the background and want to start a second one.\nIn this case ignore "
              "this message. Otherwise a previously run plotting server crashed without cleaning up afterwards. \nIn this case, please manually delete the file at {}".format(port_file_path),
              file = sys.stderr)
    with open(port_file_path, 'wb') as f:
        pickle.dump(port,f)


def remove_port_file():
    print("Removing port file")
    port_file_path = get_local_path("tmp/plot_server/port.info", make_local_dir=True)
    if os.path.exists(port_file_path):
        os.remove(port_file_path)


def save_current_figure():
    print("Attempting to save figure")
    fig = plt.gcf()
    file_name = format_filename(file_string = '%T', current_time = datetime.now())
    save_path = get_local_path('output/{file_name}.pdf'.format(file_name=file_name))
    print("Current figure saved to {}".format(save_path))
    save_figure(fig,path=save_path)


class GracefulKiller:
    kill_now = False
    def __init__(self):
        signal.signal(signal.SIGINT, self.exit_gracefully)

    def exit_gracefully(self, signum, frame):
        print("SIGINT caught")
        self.kill_now = True


def run_plotting_server(address, port):
    """
    Address and port to listen on.
    :param address:
    :param port:
    :return:
    """

    # Get the first available socket starting from portand communicate it with the client who started this server
    sock, port = get_socket(address=address, port=port)
    write_port_to_file(port)
    max_number_clients = 100
    max_plot_batch_size = 2000
    sock.listen(max_number_clients)
    print(port)
    print("Plotting Server is listening")

    # We want to save and rescue the current plot in case the plotting server receives a signal.SIGINT (2)
    killer = GracefulKiller()

    # The plotting server receives input in a queue and returns the plot_ids as a way of communicating that it has rendered the plot
    main_input_queue = Queue.Queue()
    return_queue = Queue.Queue()
    # Start accepting clients' communication requests
    t0 = threading.Thread(target=handle_socket_accepts,args=(sock, main_input_queue, return_queue, max_number_clients))
    t0.setDaemon(True)
    t0.start()

    # If killed, save the current figure
    atexit.register(save_current_figure)

    # Now, we can accept plots in the main thread!

    set_dbplot_figure_size(9,10)
    while True:
        if killer.kill_now:
            # The server has received a signal.SIGINT (2), so we stop receiving plots and terminate
            break
        # Retrieve data points that might have come in in the mean-time:
        client_messages = _queue_get_all_no_wait(main_input_queue,max_plot_batch_size)
        # client_messages is a list of ClientMessage objects
        if len(client_messages) > 0:
            return_values = []
            with hold_dbplots():
                for client_msg in client_messages:  # For each ClientMessage object
                    # Take apart the received message, plot, and return the plot_id to the client who sent it
                    plot_message = pickle.loads(client_msg.dbplot_message)  # A DBPlotMessage object (see plotting_client.py)
                    plot_message.dbplot_args['draw_now'] = False
                    axis = dbplot(**plot_message.dbplot_args)
                    axis.ticklabel_format(style='sci', useOffset=False)
                    return_values.append((client_msg.client_address, plot_message.plot_id))
                plt.rcParams.update({'axes.titlesize': 'small', 'axes.labelsize': 'small'})
                plt.subplots_adjust(hspace=0.4,wspace=0.6)
            for client, plot_id in return_values:
                return_queue.put([client,plot_id])
        else:
            time.sleep(0.1)


def _queue_get_all_no_wait(q, max_items_to_retreive):
    """
    Empties the queue, but takes maximally maxItemsToRetreive from the queue
    :param q:
    :param max_items_to_retreive:
    :return:
    """
    items = []
    for numOfItemsRetrieved in range(0, max_items_to_retreive):
        try:
            items.append(q.get_nowait())
        except Queue.Empty, e:
            break
    return items


def handle_socket_accepts(sock, main_input_queue, return_queue, max_number):
    """
    This Accepts max_number of incomming communication requests to sock and starts the threads that manages the data-transfer between the server and the clients
    :param sock:
    :param main_input_queue:
    :param return_queue:
    :param max_number:
    :return:
    """
    return_lock = threading.Lock()
    for _ in range(max_number):
        connection, client_address = sock.accept()
        t0 = threading.Thread(target=handle_input_connection,args=(connection, client_address, main_input_queue))
        t0.setDaemon(True)
        t0.start()

        t1 = threading.Thread(target=handle_return_connection,args=(connection, client_address, return_queue, return_lock))
        t1.setDaemon(True)
        t1.start()


def handle_return_connection(connection, client_address, return_queue, return_lock):
    """
    For each client, there is a thread that continously checks for the confirmation that a plot from this client has been rendered.
    This thread takes hold of the return queue, dequeues max 10 objects and checks if there is a return message for the client that is goverend by this thread.
    All other return messages are put back into the queue. Then the lock on the queue is released so that other threads might serve their clients their respecitve messages.
    The return messges belonging to this client are then sent back.
    :param connection:
    :param client_address:
    :param return_queue:
    :param return_lock:
    :return:
    """
    while True:
        return_lock.acquire()
        return_objects = _queue_get_all_no_wait(return_queue, 10)
        if len(return_objects) > 0:
            # print("Received {} return objects".format(len(return_objects)))
            owned_items = []
            for client, plot_id in return_objects:
                if client == client_address:
                    owned_items.append(plot_id)
                else:
                    return_queue.put((client,plot_id))
            return_lock.release()
            for plot_id in owned_items:
                message = plot_id
                send_size(sock=connection, data=message)
        else:
            return_lock.release()
            # print("no return value to send :)")
            time.sleep(0.01)


ClientMessage = namedtuple('ClientMessage', ['dbplot_message', 'client_address'])
# dbplot_args is a DBPlotMessage object
# client_address: A string IP address


def handle_input_connection(connection, client_address, input_queue):
    """
    For each client, there is a thread that waits for incoming plots over the network. If a plot came in, this plot is then put into the main queue from which the server takes
    plots away.
    :param connection:
    :param client_address:
    :param input_queue:
    :return:
    """
    while True:
        recv_message = recv_size(connection)
        input_queue.put(ClientMessage(recv_message, client_address))
        # input_queue.put({"plot":recv_message,"client":client_address}, block=False)
    connection.close()


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--reuse', action="store_true", help='set if you want to merge plots on this server into one shared memory')
    args = parser.parse_args()
    set_server_plotting(False)  # This causes dbplot to configure correctly
    if args.reuse is True:
        # TODO: has not been tested thoroughly yet. For example if you spawn two processes using the same server at the same time, there might be a conflict about who is the first who
        # actually sets up the server, and who joins the existing server
        send_port_if_running_and_join()

    run_plotting_server("0.0.0.0",7000) # We listen to the whole internet and start with port 7000



__author__ = 'peter'


class CheckPointCounter(object):
    """
    Scenerio: You have a loop, and want to do something periodically within that loop, but not
    on every iteration.  Use this object to tell you if you want to do that thing or not.

    You give it a series of checkpoints, and each time it is called, it will return the number
    of checkpoints it has passed since it was last called.  If you only want to know IF it's passed
    a checkpoint, you can treat the return as a boolean.
    """

    def __init__(self, checkpoints):
        self._checkpoints = checkpoints
        self._index = 0

    def check(self, progress):
        """
        :param progress: Indicator of the progress - on the same scale of the checkpoints.
        :return: (points_passed, done), where:
            points_passed is the number of checkpoints passed since the last call to this method,
            done is a boolean indicating whether the last checkpoint has been passed.

        Note that done will only be True of points_passed>0
        """
        counter = 0
        done = False

        while True:
            if self._index == len(self._checkpoints):
                done = True
                break
            elif progress < self._checkpoints[self._index]:
                break
            else:
                counter += 1
                self._index += 1

        return counter, done



from artemis.general.image_ops import resize_while_preserving_aspect_ratio, equalize_image_dims

__author__ = 'peter'
import numpy as np


def test_resize_while_preserving_aspect_ratio():

    img = np.random.randn(10, 20, 3)
    bwimg = np.random.randn(10, 20)

    img2 = resize_while_preserving_aspect_ratio(img, x_dim=30)
    assert img2.shape == (15, 30, 3)

    bwimg2 = resize_while_preserving_aspect_ratio(bwimg, x_dim=30)
    assert bwimg2.shape == (15, 30)

    img3 = resize_while_preserving_aspect_ratio(img, y_dim=5)
    assert img3.shape == (5, 10, 3)


def test_equalize_image_dims():

    imgs = [np.random.randn(12, 18, 3), np.random.randn(12, 12), np.random.randn(24, 12)]

    new_imgs = equalize_image_dims(imgs, x_dim = 'max', y_dim='max')
    assert [im.shape for im in new_imgs] == [(12, 18, 3), (18, 18), (24, 12)]

    new_imgs = equalize_image_dims(imgs, x_dim = 12, y_dim=12)
    assert [im.shape for im in new_imgs] == [(8, 12, 3), (12, 12), (12, 6)]

    new_imgs = equalize_image_dims(imgs, x_dim = 16)
    assert [im.shape for im in new_imgs] == [(11, 16, 3), (16, 16), (32, 16)]

    new_imgs = equalize_image_dims(imgs, y_dim = 16)
    assert [im.shape for im in new_imgs] == [(16, 24, 3), (16, 16), (16, 8)]

    new_imgs = equalize_image_dims(imgs, x_dim = 'max', y_dim = 16)
    assert [im.shape for im in new_imgs] == [(12, 18, 3), (16, 16), (16, 8)]


if __name__ == "__main__":
    test_resize_while_preserving_aspect_ratio()
    test_equalize_image_dims()



import datetime
from artemis.fileman.config_files import get_config_value
from artemis.config import get_artemis_config_value
import os

__author__ = 'peter'


"""
For dealing with files in a uniform way between machines, we have a local directory for data.

The idea is that be able to put things in the code like:

mnist = pkl.read(open('data/mnist.pkl'))

Where the path is referenced relative to the data directory on that machine.
"""


def get_default_local_path():
    return os.path.join(os.getenv("HOME"), '.artemis')

LOCAL_DIR = get_artemis_config_value(section='fileman', option='data_dir', default_generator = get_default_local_path, write_default = True)


def get_local_path(relative_path = '', make_local_dir = False):
    """
    Get the full local path of a file relative to the Data folder.  If the relative path starts with a "/", we consider
    it to be a local path already.  TODO: Make this Windows-friendly

    :param relative_path: A path relative to the data directory.  If it starts with "/", we consider it to be already
    :param make_local_dir: True to create the directory that the path points to, if it does not already exist.
    :return: The full path to the file
    """
    if not relative_path.startswith('/'):
        # Path is considered relative to data directory.
        file_path = os.path.join(LOCAL_DIR, format_filename(relative_path))
    else:
        file_path = relative_path
    if make_local_dir:
        make_file_dir(file_path)
    return file_path

def get_local_dir(relative_path = '', create_dir_if_not_exist=True):
    """
    Get the full path of a directory relative to the Data folder. If the relative path starts with a "/", we consider
    it to be a local path already.
    :param relative_path: A path relative to the data directory.  If it starts with "/", we consider it to be already a local path
    :param create_dir_if_not_exist: True to create the directory that the path points to, if it does not already exist.
    :return: The full path to the directory
    """
    if not relative_path.endswith("/"):
        relative_path += "/"
    return get_local_path(relative_path=relative_path, make_local_dir=create_dir_if_not_exist)

def get_relative_path(local_path, base_path = LOCAL_DIR):
    assert local_path.startswith(base_path), '"%s" is not contained within the data directory "%s"' % (local_path, base_path)
    relative_loc = local_path[len(base_path)+1:]
    return relative_loc


def make_file_dir(full_file_path):
    """
    Make the directory containing the file in the given path, if it doesn't already exist
    :param full_file_path:
    :returns: The directory.
    """
    full_local_dir, _ = os.path.split(full_file_path)
    try:
        os.makedirs(full_local_dir)
    except OSError:
        pass
    return full_file_path


def make_dir(full_dir):
    try:
        os.makedirs(full_dir)
    except OSError:
        pass
    return full_dir


_ALPHANUMERICS = [chr(a) for a in xrange(ord('a'), ord('z')+1)]+[chr(a) for a in xrange(ord('0'), ord('9')+1)]


def format_filename(file_string, current_time = 'now', base_name = None, directory = None, ext = None, allow_partial_formatting = False):
    """
    Return a formatted string with placeholders in the filestring replaced by their provided values.
    :param file_string: A string, eg '%T-%N'.  The placeholders %T, %N indicate that they should be replaced
        with the time and the provided name, respectively.
        The following placeholders can be used:
            %T: Replace with the time (will look like e.g. 2016.05.20T04.23.53.145988
            %R: Replace with a random alphanumeric character 'a'-'z', '0'-'9'
    :param current_time: Current time, as returned by datetime.datetime.now() - the ISO representation of this
        time will be used to fill the %T placeholder
    :param base_name: The name to swap in for the %N placeholder
    :param rel_dir: Optionally, a directory to prepend to the file_string, relative to the local storage folder (if any).
        This can also contain placeholders
    :param local__dir: Optionally, the local folder on this machine where you store your data.  This defaults to the
        directory returned by get_local_path
    :param ext: Optionally, an extension to append to the filename.
    :param allow_partial_formatting: If True, the placeholders (%T, %N) are allowed to pass through if no values are
        specified.  This may be useful if one placeholder is not yet known at the time the rest of the info is specified.
    :return: The formatted filename.
    """

    if directory is not None:
        file_string = os.path.join(directory, file_string)

    if ext is not None:
        file_string += '.'+ext

    if '%T' in file_string:
        if current_time == 'now':
            current_time = datetime.datetime.now()
        if current_time is not None:
            iso_time = current_time.isoformat().replace(':', '.').replace('-', '.')
            file_string = file_string.replace('%T', iso_time)
        else:
            raise Exception('You failed to specify a valid time, or string "now"')
    if '%R' in file_string:
        from random import Random
        rng = Random()
        while '%R' in file_string:
            file_string = file_string.replace('%R', rng.choice(_ALPHANUMERICS), 1)

    if '%N' in file_string:
        if base_name is None:
            assert allow_partial_formatting, 'You included "%N" in the file string but had no base_name argument'
        else:
            file_string = file_string.replace('%N', base_name)

    return file_string



from artemis.ml.tools.iteration import minibatch_index_generator, checkpoint_minibatch_index_generator, \
    zip_minibatch_iterate_info

__author__ = 'peter'
import numpy as np


def test_minibatch_index_generator():

    n_samples = 48
    n_epochs = 1.5
    minibatch_size = 5

    data = np.arange(n_samples)

    expected_total_samples = int(len(data)*n_epochs)

    for slice_when_possible in (True, False):

        i = 0
        for ix in minibatch_index_generator(n_samples = n_samples, n_epochs=n_epochs, final_treatment='truncate',
                slice_when_possible = slice_when_possible, minibatch_size=minibatch_size):
            assert np.array_equal(data[ix], np.arange(i, min(expected_total_samples, i+minibatch_size)) % n_samples)
            i += len(data[ix])
        assert i == expected_total_samples == 72

        i = 0
        for ix in minibatch_index_generator(n_samples = n_samples, n_epochs=n_epochs, final_treatment='stop',
                slice_when_possible = slice_when_possible, minibatch_size=minibatch_size):
            assert np.array_equal(data[ix], np.arange(i, min(expected_total_samples, i+minibatch_size)) % n_samples)
            i += len(data[ix])
        assert i == int(expected_total_samples/minibatch_size) * minibatch_size == 70


def test_checkpoint_minibatch_generator():
    n_samples = 48
    data = np.arange(n_samples)
    for checkpoints in ([0, 20, 30, 63, 100], [20, 30, 63, 100]):
        for slice_when_possible in (True, False):
            iterator = checkpoint_minibatch_index_generator(n_samples=n_samples, checkpoints=checkpoints, slice_when_possible=slice_when_possible)
            assert np.array_equal(data[iterator.next()], np.arange(20))
            assert np.array_equal(data[iterator.next()], np.arange(20, 30))
            assert np.array_equal(data[iterator.next()], np.arange(30, 63) % 48)
            assert np.array_equal(data[iterator.next()], np.arange(63, 100) % 48)
            try:
                iterator.next()
            except StopIteration:
                pass
            except:
                raise Exception("Failed to stop iteration")


def test_minibatch_iterate_info():

    n_samples = 100
    minibatch_size = 10

    training_arr = np.random.randn(n_samples, 12)
    test_arr = np.random.randint(10, size=n_samples)

    for kwargs in [dict(test_epochs=[0, 0.5, 1, 1.5]), dict(test_epochs=('every', 0.5), n_epochs=1.5)]:
        # Two different ways of parametrixing minibatch loop:
        # test_epochs=[0, 0.5, 1, 1.5]:  "Test at these epochs.  Done after final test"
        # test_epochs=('every', 0.5), n_epochs=1.5  : "Test every 0.5 epochs.  Done after test at epcoh 1.5

        test_epochs = []
        iterator = 0

        for (training_minibatch, test_minibatch), info in zip_minibatch_iterate_info((training_arr, test_arr),
                    minibatch_size=minibatch_size, **kwargs):

            ixs = (np.arange(minibatch_size)+iterator*minibatch_size) % n_samples
            epoch = iterator * minibatch_size / float(n_samples)
            print (epoch, info.epoch)
            print info.done
            assert np.allclose(epoch, info.epoch)
            assert np.array_equal(ixs, np.arange(info.sample, info.sample+minibatch_size) % n_samples)
            assert np.array_equal(training_minibatch, training_arr[ixs])
            assert np.array_equal(test_minibatch, test_arr[ixs])

            if info.test_now:
                test_epochs.append(info.epoch)

            if not info.done:
                iterator += 1

        assert iterator == 15
        assert test_epochs == [0, 0.5, 1, 1.5]  # Note... last test time is not included... difficult to say what should be expected here.


if __name__ == '__main__':
    test_minibatch_iterate_info()
    test_minibatch_index_generator()
    test_checkpoint_minibatch_generator()



import sys
import textwrap
from StringIO import StringIO
from contextlib import contextmanager

from artemis.fileman.local_dir import make_file_dir

__author__ = 'peter'
import numpy as np


def deeprint(obj, memo=None):
    """
    Consise - print.

    TODO: Extend this to make a proper deep-print of any object.
    """
    if isinstance(obj, np.ndarray):
        string = '<%s with shape=%s, dtype=%s at %s%s>' % (obj.__class__.__name__, obj.shape, obj.dtype, hex(id(obj)),
            ', value = '+str(obj) if obj.size<8 else ''
            )
    else:
        string = str(obj)
    return string


_ORIGINAL_STDOUT = sys.stdout
_ORIGINAL_STDERR = sys.stderr


class CaptureStdOut(object):
    """
    An logger that both prints to stdout and writes to file.
    """

    def __init__(self, log_file_path = None, print_to_console = True):
        """
        :param log_file_path: The path to save the records, or None if you just want to keep it in memory
        :param print_to_console:
        """
        self._print_to_console = print_to_console
        if log_file_path is not None:
            # self._log_file_path = os.path.join(base_dir, log_file_path.replace('%T', now))
            make_file_dir(log_file_path)
            self.log = open(log_file_path, 'w')
        else:
            self.log = StringIO()
        self._log_file_path = log_file_path
        self.terminal = _ORIGINAL_STDOUT

    def __enter__(self):
        sys.stdout = self
        sys.stderr = self
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        sys.stdout = _ORIGINAL_STDOUT
        sys.stderr = _ORIGINAL_STDERR
        self.close()

    def get_log_file_path(self):
        assert self._log_file_path is not None, "You never specified a path when you created this logger, so don't come back and ask for one now"
        return self._log_file_path

    def write(self, message):
        if self._print_to_console:
            self.terminal.write(message)
        self.log.write(message)
        self.log.flush()

    def close(self):
        if self._log_file_path is not None:
            self.log.close()

    def read(self):
        if self._log_file_path is None:
            return self.log.getvalue()
        else:
            with open(self._log_file_path) as f:
                txt = f.read()
            return txt

    def __getattr__(self, item):
        return getattr(self.terminal, item)


class IndentPrint(object):
    """
    Indent all print statements
    """

    def __init__(self, block_header=None, spacing = 4, show_line = False, show_end = False):
        self.indent = '|'+' '*(spacing-1) if show_line else ' '*spacing
        self.show_end = show_end
        self.block_header = block_header

    def __enter__(self):
        self.old_stdout = sys.stdout
        if self.block_header is not None:
            print self.block_header
        sys.stdout = self

    def flush(self):
        self.old_stdout.flush()

    def write(self, message):
        if message=='\n':
            new_message = '\n'
        else:
            new_message = self.indent + message.replace('\n', '\n'+self.indent)
        self.old_stdout.write(new_message)

    def close(self):
        self.old_stdout.close()

    def __exit__(self, exc_type, exc_val, exc_tb):
        sys.stdout = self.old_stdout
        if self.show_end:
            print '```'


def side_by_side(multiline_strings, gap=4, max_linewidth=None):
    """
    Return a string that displays two multiline strings side-by-side.
    :param multiline_strings:
    :param gap:
    :param max_linewidth:
    :return:
    """
    if max_linewidth is not None:
        multiline_strings = [textwrap.fill(s, width=max_linewidth, replace_whitespace=False) for s in multiline_strings]

    lineses = [s.split('\n') for s in multiline_strings]
    longests = [max(len(line) for line in lines) for lines in lineses]

    spacer = ' '*gap
    new_lines = []
    for i in xrange(max(len(lines) for lines in lineses)):
        line = [lines[i] if i<len(lines) else '' for lines in lineses]
        final_line = [line + ' '*(max_length-len(line)) for line, max_length in zip(line, longests)]
        new_lines.append(spacer.join(final_line))
    return '\n'.join(new_lines)




from artemis.general.should_be_builtins import bad_value
from artemis.general.tables import build_table


def test_build_table(show_table=True):

    def lookup_function(prisoner_a_choice, prisoner_b_choice):
        total_utility = \
            2 if prisoner_a_choice=='cooperate' and prisoner_b_choice=='cooperate' else \
            3 if prisoner_a_choice != prisoner_b_choice else \
            4 if prisoner_b_choice=='betray' and prisoner_a_choice=='betray' \
            else bad_value((prisoner_a_choice, prisoner_b_choice))
        return total_utility

    rows = build_table(lookup_function, row_categories=['cooperate', 'betray'], column_categories=['cooperate', 'betray'])

    assert rows == [[' ', 'Cooperate', 'Betray'], ['Cooperate', 2, 3], ['Betray', 3, 4]]

    if show_table:
        import tabulate
        print tabulate.tabulate(rows)


if __name__ == '__main__':
    test_build_table()



from datetime import datetime
import sys
from artemis.fileman.local_dir import get_local_path, get_relative_path
import os

__author__ = 'peter'


def get_local_server_dir(subdir = None):
    """
    Get the directory at the root of the venv.
    :param subdir:
    :return:
    """
    figures_dir = os.path.abspath(os.path.join(sys.executable, '..', '..', '..'))
    if subdir is not None:
        figures_dir = os.path.join(figures_dir, subdir)
    return figures_dir


DATA_FOLDER_NAME = 'Data'
SERVER_RELATIVE_DATA_DIR = os.path.join(get_local_server_dir(), DATA_FOLDER_NAME)


def get_server_relative_data_folder_name():
    return DATA_FOLDER_NAME


def get_relative_link_from_local_path(local_path):
    """
    Given an abolulute path on the local machine, return a relative link to the file so that
    you can access it from the server.
    :param local_path: A local path.
    :return: A relative path
    """
    relative_path = get_relative_path(local_path)
    return get_relative_link_from_relative_path(relative_path)


def get_relative_link_from_relative_path(relative_path):
    """
    Given a local path to a file in the data folder, return the relative link that will access it from
    the server.

    To do this, we make a soft-link from the server directory to the data folder - this way we can
    browse our data folder from Jupyter, which is nice.

    :param relative_path: Relative path (from within Data folder)
    :return: A string representing the relative link to get to that file.
    """
    true_local_data_dir = get_local_path()

    local_path = get_local_path(relative_path)
    launcher = 'tree' if os.path.isdir(local_path) else 'files'

    if not os.path.lexists(SERVER_RELATIVE_DATA_DIR):
        os.symlink(true_local_data_dir, SERVER_RELATIVE_DATA_DIR)
    return os.path.join('/', launcher, DATA_FOLDER_NAME, relative_path)






import os
from artemis.fileman.config_files import get_config_path, get_config_value

_DEFAULT_ARTEMIS_CONFIG = """
[plotting]
backend = matplotlib
update_period = 0.5
mode = safe
default_fig_size = (12, 8)
"""


def get_artemis_config_value(section, option, default_generator = None, write_default = False, read_method=None):
    """
    Get a setting from the artemis configuration.
    See docstring for get_config_value
    """
    config_path = get_config_path('.artemisrc')
    if not os.path.exists(config_path):
        with open(config_path, 'w') as f:
            f.write(_DEFAULT_ARTEMIS_CONFIG)
    return get_config_value('.artemisrc', section=section, option=option, default_generator=default_generator, write_default=write_default, read_method = read_method)



from itertools import izip
from artemis.fileman.file_getter import get_file, unzip_gz
from artemis.fileman.smart_io import smart_load
import numpy as np
import os
__author__ = 'peter'


def get_imagenet_fall11_urls(n_images = None):

    if n_images is None:
        n_images = 14197121

    imagenet_urls = get_file(
        'data/imagnet_urls.txt',
        url = 'http://image-net.org/imagenet_data/urls/imagenet_fall11_urls.tgz',
        data_transformation=unzip_gz
        )

    print 'Loading %s image URLs....' % (n_images, )
    with open(imagenet_urls) as f:
        f.readline()
        lines = list(line for _, line in izip(xrange(n_images), f))
    indices = [s.index('\t') for s in lines]
    pairs = [(line[:s], line[s+1:-1]) for line, s in zip(lines, indices)]
    print 'Done.'
    return pairs


def get_imagenet_images(indices):
    """
    Get imagenet images at the given indices
    :param indices:
    :return:
    """
    highest_index = np.max(indices)
    code_url_pairs = get_imagenet_fall11_urls(highest_index+1)
    files = [get_file('data/imagenet/%s%s' % (code_url_pairs[index][0], os.path.splitext(code_url_pairs[index][1])[1]), code_url_pairs[index][1]) for index in indices]
    return [smart_load(f) for f in files]


if __name__ == '__main__':
    # Downloads 4 random images out of the first 1000.  You may get 404 errors, etc.  So just run again and again til this works.
    import random
    from artemis.plotting.db_plotting import dbplot
    ixs = [random.randint(0, 999) for _ in xrange(4)]
    print ixs
    ims = get_imagenet_images(ixs)
    for i, (ix, im) in enumerate(zip(ixs, ims)):
        dbplot(im, 'Image %s' % i, hang = i==len(ims)-1)



from artemis.general.nested_structures import flatten_struct, get_meta_object, NestedType
import numpy as np
from pytest import raises


def test_flatten_struct():

    a = {
        'a': [1, 2, 3],
        'b': {
            'c': 4,
            'd': {'e': 5, 6: [7, 8]},
            },
        'c': 1
        }
    b = dict(flatten_struct(a))
    assert b["['a'][1]"]==2
    assert b["['b']['d'][6][1]"]==8

    class A(object):

        def __init__(self, other_object, b):
            self._a = other_object
            self._b = b

    thing = A(A(3, 'arr'), {'x': np.random.randn(3, 2), 'y': (3, 4, 5)})

    flat_thing = dict(flatten_struct(thing))

    assert flat_thing['._a._b']=='arr'
    assert flat_thing["._b['y'][0]"]==3


_complex_obj = [1, 2, {'a':(3, 4), 'b':['hey', 'yeah']}, 'woo']
_meta_obj = [int, int, {'a':(int, int), 'b':[str, str]}, str]


def test_get_meta_object():
    assert get_meta_object(_complex_obj) == [int, int, {'a':(int, int), 'b':[str, str]}, str]


def test_nested_type():
    # Example from docstring
    assert NestedType.from_data([1, 2, {'a': (3, 4.), 'b': 'c'}]) == NestedType([int, int, {'a': (int, float), 'b': str}])

    # Other tests
    nested_type = NestedType.from_data(_complex_obj)
    assert nested_type.is_type_for(_complex_obj)
    _complex_obj_2 = [1, 2, {'a': (3, 4, 5), 'b': ['hey', 'yeah']}, 'woo']
    _complex_obj_3 = [1, 2, {'a': (3, 5), 'b': ['hey', 'yeah']}, 'woo']
    assert not nested_type.is_type_for(_complex_obj_2)
    assert nested_type.is_type_for(_complex_obj_3)
    assert nested_type == NestedType(_meta_obj)
    leaves = [1, 2, 3, 4, 'hey', 'yeah', 'woo']
    assert nested_type.get_leaves(_complex_obj) == leaves
    assert nested_type.expand_from_leaves(leaves) == _complex_obj

    extended_leaves = leaves + ['x']
    with raises(TypeError):
        nested_type.expand_from_leaves(extended_leaves)

    # Change the type
    swapped_leaves = list(leaves)
    swapped_leaves[0] = 'eh?'
    with raises(TypeError):
        nested_type.expand_from_leaves(swapped_leaves)


if __name__ == '__main__':
    test_flatten_struct()
    test_get_meta_object()
    test_nested_type()



from itertools import count
from artemis.general.checkpoint_counter import CheckPointCounter

__author__ = 'peter'


def test_checkpoint_counter():

    points = [0, 0.2, 0.4, 0.6, 0.9]

    cpc = CheckPointCounter(points)

    checkpoints = []
    for i in count(0):
        points_passed, done = cpc.check(i/100.)
        if points_passed:
            checkpoints.append(i)
            if done:
                break

    assert len(checkpoints)==len(points)
    assert checkpoints == [0, 20, 40, 60, 90]
    # Only because all points are spaced wider than progress increments.
    # More generally len(checkpoints) will be <= len(points)


if __name__ == '__main__':
    test_checkpoint_counter()
